/* mini-at/src/mini-at/main.c
 *
 *  (c)2006, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv2. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



void version(void)
{
    printf("mini-at " VERSION "\n"
        "(c)2006, Laurence Withers.\n");
}



void usage(void)
{
    printf("Usage:\n\n"
        "    mini-at [options] time cmd [args ...]\n\n"
        "Options:\n"
        "-h, --help         Display this screen.\n"
        "-V, --version      Display version.\n"
        "-f, --file <x>     Specify filename to read stdin from, - to copy stdin.\n",
        "-0, --argv0 <x>    Change argv[0] from `cmd' to `x'.\n"
        "\n"
        "'time' is an ISO8601 time. If you specify an incomplete time, e.g. '2010', we simply\n"
        "take the earliest time it could represent (i.e. 2010-01-01T00:00:00).\n"
        "\n");
    version();
}



struct option options[] = {
    { "help", no_argument, 0, 'h' },
    { "version", no_argument, 0, 'V' },
    { "file", required_argument, 0, 'f' },
    { "argv0", required_argument, 0, '0' },
    { 0, 0, 0, 0 }
}



int main(int argc, char* argv[])
{
    const char* stdin_file = 0, * argv0 = 0;
    struct at_msg a;
    char msg[AT_MSG_SIZE];
    int size, i, x;

    while(1) {
        switch(getopt_long(argc, argv, "hVf:0:", options, 0)) {
        case -1:
            goto done;

        case 'h':
            usage();
            return 0;

        case 'V':
            version();
            return 0;

        case 'f':
            if(stdin_file) {
                fprintf(stderr, "Input file can only be specified once.\n");
                return 1;
            }
            stdin_file = optarg;
            break;

        case '0':
            if(argv0) {
                fprintf(stderr, "argv[0] can only be specified once.\n");
                return 1;
            }
            argv0 = optarg;
            break;
        }
    }

done:
    argv += optind;
    argc -= optind;
    if(argc < 2) {
        usage();
        return 1;
    }

    // parse time
    t = parse_time(argv[0]);
    if(t == (time_t)-1) return 1;
    ++argv;
    --argc;

    // set up message header
    memset(&a, 0, sizeof(a));
    a.magic = AT_MAGIC;
    a.version = (VERMAJOR << 20) | (VERMINOR << 10) | VERMICRO;
    a.when = t;
    a.cmd_len = strlen(argv[0]);
    if(stdin_file) {
        stdin_file = do_stdin(stdin_file);
        a.in_fn_len = strlen(stdin_file);
    }
    a.num_args = argc;

    // set up message
    size = 0;
    msg_add(msg, &size, &a, sizeof(a));
    msg_add(msg, &size, argv[0], a.cmd_len);
    if(stdin_file) msg_add(msg, &size, stdin_file, a.in_fn_len);

    if(!argv0) argv0 = argv[0];
    x = strlen(argv0);
    msg_add(msg, &size, &x, sizeof(x));
    msg_add(msg, &size, argv0, x);

    for(i = 1; i < argc; ++i) {
        x = strlen(argv[i]);
        msg_add(msg, &size, &x, sizeof(x));
        msg_add(msg, &size, argv[i], x);
    }

    // send message
    // TODO

    return 0;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
